/* ColorChanger
 * 
 * Reads in an RGB value from the serial port and lights up an RGB LED the appropriate color.
 * The value must be passed in as a hex RGB value in the form of "#RRGGBBSS" where:
 *  RR: red value 
 *  GG: green value
 *  BB: blue value
 *  SS: LED state
 * The state value (SS) can be "00" for solid, or "01" for throb.
 *     Example1: #FF00FF01
 *     - RGB LED will throb purple
 *
 *     Example2: #4169E100
 *     - LED will glow a solid royal blue
 * 
 * This code utilizes the PWM pins (9, 10, 11) on the Arduino to light the LED.  This allows
 * the LED to either glow solid or throb (gradually fade in and out).
 * noise).  
 *
 * Nithin Shenoy
 * nithin_shenoy@hotmail.com
 * 2009-04-09
 *
 * http://yaam.codeplex.com
 * http://nithin.spaces.live.com
 */
 
#define LED_RED    9
#define LED_BLUE   10
#define LED_GREEN  11

#define STATE_SOLID 0x0000
#define STATE_THROB 0x0001
#define STATE_BLINK 0x0010

byte serialVal = 0;
byte rCol = 0;
byte gCol = 0;
byte bCol = 0;
byte state = 0;
char buffer[8];
byte idx=0;

// Taken from "Getting Started with Arduino", Massimo Banzi, page 82
byte hex2dec(byte c)
{
 if (c >= '0' && c <= '9')
 {
  return c - '0';
 }
 else if (c >= 'A' && c <= 'F')
 {
   return c - 'A' + 10;
 }
}

// Lights the LED a solid RGB value
void solid(byte redVal, byte greenVal, byte blueVal)
{
   analogWrite(LED_RED, redVal);
   analogWrite(LED_GREEN, greenVal);
   analogWrite(LED_BLUE, blueVal);
}

// Causes LED to throb the RGB value by gradually fading in an out
void throb(byte redVal, byte greenVal, byte blueVal)
{
  float maxVal = 255;
  byte i = 0;
  bool countDown = false;

  do
  {
    solid((byte)(i*(redVal/maxVal)), (byte)(i*(greenVal/maxVal)), (byte)(i*(blueVal/maxVal)));
    
    if (true == countDown)
    {
      delay(5);
      i--;
    }
    else
    {
      delay(10);
      i++;
      
      if (255 == i)
      {
        countDown = true;
      }
    }
  } while (i>0);

  solid(0,0,0);
  delay(5);
}

void setup()
{
  pinMode(LED_RED, OUTPUT);
  pinMode(LED_BLUE, OUTPUT);
  pinMode(LED_GREEN, OUTPUT);
  Serial.begin(9600);
}

void loop()
{
  if (Serial.available()>0)
  {
    serialVal = Serial.read();
    
    // Required to prevent corruption in the buffer. No idea why.
    delay(10);  
    
    if (serialVal == '#')
    {
      while(idx<8)
      {
       buffer[idx] = Serial.read();
       idx++;
      }

      idx = 0;
     
      // Convert characters to RGB values
      rCol = hex2dec(buffer[1]) + hex2dec(buffer[0])*16;
      gCol = hex2dec(buffer[3]) + hex2dec(buffer[2])*16;
      bCol = hex2dec(buffer[5]) + hex2dec(buffer[4])*16;
      state = hex2dec(buffer[7]) + hex2dec(buffer[6])*16;

#ifdef _DEBUG      
      Serial.print("  red=");
      Serial.println(rCol,HEX);
      Serial.print("  green=");
      Serial.println(gCol,HEX);
      Serial.print("  blue=");
      Serial.println(bCol,HEX);
      Serial.print("  option=");
      Serial.println(state,HEX);
#endif

    }
  }
  
  // Finite state machine to control LED behavior
  switch (state)
  {
    case STATE_SOLID:
      solid(rCol, gCol, bCol);
      break;
    case STATE_THROB:
      throb(rCol, gCol, bCol);
      break;
    default:
      state = STATE_SOLID;
  }

}
